module Core.Handlers where

import qualified Core.Configuration      as Conf
import           Core.Feed               (Post(..), autoFeed, renderFeed)
import           Core.Rendering 
import           Core.Styles             as S
import           Fragments.Base
import           Fragments.NotFound

import qualified Text.Atom.Feed          as Atom
import           Control.Monad.IO.Class  (liftIO)
import           Data.Text
import qualified Data.Text.Lazy.Encoding as LTE
import           Data.Time.Clock         (UTCTime(..), getCurrentTime)
import           Data.Time.Format        (formatTime, defaultTimeLocale)
import           Lucid                   (Html)
import           System.Directory        (getModificationTime)
import           Web.Twain

-- A ResponoderM capable of lifting to IO monad; constructs response to clients
index :: ResponderM a
index = do 
  -- Query the system environment for the BLOGTITLE environment variable
  title    <- liftIO   Conf.appTitle
  -- Read a Commonmark Markdown file and process it to HTML
  homeMd   <- liftIO $ mdFileToLucid "./data/posts/home.md"
  -- Get the Footer content from Markdown
  footerMd <- liftIO $ mdFileToLucid "./data/posts/footer.md"
  -- Respond to request with fragments compositionally to create a home page
  sendLucidFragment  $ basePage title (baseHome homeMd) footerMd

-- Responds with processed Commonmark -> HTML for posts existing at app init
posts :: FilePath -> ResponderM a 
posts postName = do
  title    <- liftIO Conf.appTitle
  footerMd <- liftIO $ mdFileToLucid "./data/posts/footer.md"
  postMd   <- liftIO $ mdFileToLucid ("./data/posts/" ++ postName ++ ".md")
  sendLucidFragment  $ basePage title (basePost postMd) footerMd

-- Builds an index of all posts on filesystem as of application init
postsIndex :: [FilePath] -> ResponderM a
postsIndex postNames = do
  title    <- liftIO Conf.appTitle
  footerMd <- liftIO $ mdFileToLucid "./data/posts/footer.md"
  sendLucidFragment  $ basePage title (postIndex postNames) footerMd

-- Generates the XML feed at /feed
feed :: [FilePath] -> ResponderM a
feed postNames = do
  title     <- liftIO   Conf.appTitle
  time      <- liftIO $ fmap (\x -> timeFormat x) getCurrentTime
  -- Create Atom [Post] to populate the feed
  feedData  <- liftIO $ mapM makePost postNames
  -- Send an XML response with an automatically populated Atom feed
  send $ xml $ LTE.encodeUtf8 $ renderFeed 
       $ autoFeed (baseFeed title time) feedData
    where
      -- Base feed data structure which we populate with entries
      baseFeed :: String -> String -> Atom.Feed
      baseFeed title time = Atom.nullFeed
        "https://eversole.co/feed"
        (Atom.TextString $ pack title)
        (pack $ time ++ " UTC")

      -- Create an Atom Post for each markdown post present
      makePost :: FilePath -> IO (Post)
      makePost x = do 
        date   <- getModificationTime $ "./data/posts/" ++ x ++ ".md"
        return  $ Post 
          (pack $ (timeFormat date) ++ " UTC")
          (pack $ "https://eversole.co/posts/" ++ x) 
          (pack $ show x)

      -- YYYY-MM-DD HH:MM | 2024-02-24 16:36
      timeFormat :: UTCTime -> String
      timeFormat x = formatTime defaultTimeLocale "%Y-%m-%d %H:%M" x

-- Refer to index comments
contact :: ResponderM a
contact = do
  title     <- liftIO Conf.appTitle
  contactMd <- liftIO $ mdFileToLucid "./data/posts/contact.md"
  footerMd  <- liftIO $ mdFileToLucid "./data/posts/footer.md"
  sendLucidFragment   $ basePage title (baseContact contactMd) footerMd

-- Respond with primary processed CSS
theme :: ResponderM a
theme = send $ css $ LTE.encodeUtf8 $ S.cssRender S.composedStyles

-- Helper function for responding in ResponderM from Html 
sendLucidFragment :: Html () -> ResponderM a
sendLucidFragment x = send $ html $ lucidToTwain x

-- 404 handler
missing :: ResponderM a
missing = sendLucidFragment pageNotFound
